# -*- coding: utf_8 -*-

from pandac.PandaModules import *
from direct.task import Task
from math import pi

class gameTank(object):
	
	def __init__(self, model, pos, nick, mask, speed):
		self.Zold = 0
		self.angleTank = 0
		#
		self.model = model
		self.pos = pos
		self.nick = nick
		self.mask = mask
		self.speed = speed
		self.startSpeed = self.speed*1.0
		#
		self.loadModel()
		#self.loadLabel()
		self.createCollisions()
		#
		taskMgr.add(self.moveTANK, 'move-task-TANK')

	def loadModel(self):
		tX,tY,tZ,tH,tP,tR=self.pos
		# пустая нода игрока для управления
		self.node = NodePath('player')
		self.node.reparentTo(render)
		#self.node.setScale(1)
		#устанавливаем маску для коллизии
		self.node.setCollideMask(BitMask32(2))
		# модель танка
		self.model=loader.loadModel(self.model)
		self.model.reparentTo(self.node)
		self.model.setScale(0.0003,0.0003,0.0003)
		
		self.node.setPos(tX,tY,tZ)
		self.node.setHpr(tH,tP,tR)
		
		# разносим танк на детали
		self.stvol = self.model.find('**/**/stvol')
		self.bashnya = self.model.find('**/**/bashnya')
		self.a = self.node.getH()
		self.Zold = self.node.getZ()
		
		# определяем водителя для камеры		
		self.boxCam = render.attachNewNode('boxCam') 
		
	def loadLabel(self):
		#добавляем текст над танком
		font = loader.loadFont('arial.ttf')
		newTextNode = TextNode('text')
		newTextNode.setFont(font)
		newTextNode.setAlign(TextNode.ACenter)
		newTextNode.setText(self.nick)
		newTextNode.setWordwrap(16.0)
		newTextNode.setShadow(0.05,0.05)
		newTextNode.setShadowColor(1, 1, 1, 0.5)
		newTextNode.setTextColor(0,0,0,1)
		newTextNode.setFrameColor(0, 0, 1, 0.3)
		newTextNode.setFrameAsMargin(0.3, 0.3, 0.3, 0.3)
		newTextNode.setCardColor(1, 0, 0, 0.7)
		newTextNode.setCardAsMargin(0.3, 0.3, 0.3, 0.3)
		newTextNode.setCardDecal(True)
		text_generate = newTextNode.generate()
		self.newTextNodePath = render.attachNewNode(text_generate)
		self.newTextNodePath.setScale(0.08)
		self.newTextNodePath.reparentTo(self.node)
		self.newTextNodePath.setZ(0.9)
		self.newTextNodePath.setBillboardPointEye()


	def createCollisions(self):
		# луч для коллизии с полом и нахождения нормали к поверхности
		# 1
		self.tankGroundCol1 = CollisionNode('tankRay')
		self.tankGroundCol1.addSolid(CollisionRay(0,0,10,0,0,-1))
		self.tankGroundCol1.setFromCollideMask(BitMask32.bit(self.mask))
		self.tankGroundCol1.setIntoCollideMask(BitMask32.allOff())
		self.tankGroundColNp1 = self.node.attachNewNode(self.tankGroundCol1)
		self.tankGroundHandler1 = CollisionHandlerQueue()
		base.cTrav.addCollider(self.tankGroundColNp1, self.tankGroundHandler1)
		
		### 1
		self.lineCol1 = CollisionNode('tankLine1')
		self.lineCol1.addSolid(CollisionSegment(0.5, 0.8, 0.25, 0.7, 1, 0.25))
		self.lineCol1.setFromCollideMask(BitMask32.bit(self.mask))
		self.lineCol1.setIntoCollideMask(BitMask32.allOff())
		self.lineColNp1 = self.node.attachNewNode(self.lineCol1)
		self.lineHandler1 = CollisionHandlerQueue()
		base.cTrav.addCollider(self.lineColNp1, self.lineHandler1)

		### 2
		self.lineCol2 = CollisionNode('tankLine2')
		self.lineCol2.addSolid(CollisionSegment(0, 0.8, 0.25, 0, 1, 0.25))
		self.lineCol2.setFromCollideMask(BitMask32.bit(self.mask))
		self.lineCol2.setIntoCollideMask(BitMask32.allOff())
		self.lineColNp2 = self.node.attachNewNode(self.lineCol2)
		self.lineHandler2 = CollisionHandlerQueue()
		base.cTrav.addCollider(self.lineColNp2, self.lineHandler2)
		
		### 3
		self.lineCol3 = CollisionNode('tankLine3')
		self.lineCol3.addSolid(CollisionSegment(-0.5, 0.8, 0.25, -0.7, 1, 0.25))
		self.lineCol3.setFromCollideMask(BitMask32.bit(self.mask))
		self.lineCol3.setIntoCollideMask(BitMask32.allOff())
		self.lineColNp3 = self.node.attachNewNode(self.lineCol3)
		self.lineHandler3 = CollisionHandlerQueue()
		base.cTrav.addCollider(self.lineColNp3, self.lineHandler3)
		
		### 4
		self.lineCol4 = CollisionNode('tankLine4')
		self.lineCol4.addSolid(CollisionSegment(-0.5, 0, 0.25, -0.7, 0, 0.25))
		self.lineCol4.setFromCollideMask(BitMask32.bit(self.mask))
		self.lineCol4.setIntoCollideMask(BitMask32.allOff())
		self.lineColNp4 = self.node.attachNewNode(self.lineCol4)
		self.lineHandler4 = CollisionHandlerQueue()
		base.cTrav.addCollider(self.lineColNp4, self.lineHandler4)
		
		### 5
		self.lineCol5 = CollisionNode('tankLine5')
		self.lineCol5.addSolid(CollisionSegment(-0.5, -0.8, 0.25, -0.7, -1, 0.25))
		self.lineCol5.setFromCollideMask(BitMask32.bit(self.mask))
		self.lineCol5.setIntoCollideMask(BitMask32.allOff())
		self.lineColNp5 = self.node.attachNewNode(self.lineCol5)
		self.lineHandler5 = CollisionHandlerQueue()
		base.cTrav.addCollider(self.lineColNp5, self.lineHandler5)
		
		### 6
		self.lineCol6 = CollisionNode('tankLine6')
		self.lineCol6.addSolid(CollisionSegment(0, -0.8, 0.25, 0, -1, 0.25))
		self.lineCol6.setFromCollideMask(BitMask32.bit(self.mask))
		self.lineCol6.setIntoCollideMask(BitMask32.allOff())
		self.lineColNp6 = self.node.attachNewNode(self.lineCol6)
		self.lineHandler6 = CollisionHandlerQueue()
		base.cTrav.addCollider(self.lineColNp6, self.lineHandler6)
		
		### 7
		self.lineCol7 = CollisionNode('tankLine7')
		self.lineCol7.addSolid(CollisionSegment(0.5, -0.8, 0.25, 0.7, -1, 0.25))
		self.lineCol7.setFromCollideMask(BitMask32.bit(self.mask))
		self.lineCol7.setIntoCollideMask(BitMask32.allOff())
		self.lineColNp7 = self.node.attachNewNode(self.lineCol7)
		self.lineHandler7 = CollisionHandlerQueue()
		base.cTrav.addCollider(self.lineColNp7, self.lineHandler7)
		
		### 8
		self.lineCol8 = CollisionNode('tankLine8')
		self.lineCol8.addSolid(CollisionSegment(0.5, 0, 0.25, 0.7, 0, 0.25))
		self.lineCol8.setFromCollideMask(BitMask32.bit(self.mask))
		self.lineCol8.setIntoCollideMask(BitMask32.allOff())
		self.lineColNp8 = self.node.attachNewNode(self.lineCol8)
		self.lineHandler8 = CollisionHandlerQueue()
		base.cTrav.addCollider(self.lineColNp8, self.lineHandler8)
		
		###########
		"""
		self.tankGroundColNp1.show()
		
		self.lineColNp1.show()
		self.lineColNp2.show()
		self.lineColNp3.show()
		self.lineColNp4.show()
		self.lineColNp5.show()
		self.lineColNp6.show()
		self.lineColNp7.show()
		self.lineColNp8.show()
		"""
		
		
	def moveTANK(self, task):
		elapsed = globalClock.getDt()
		#########################
		# вычисление боковую коллизию танка 1
		entries = []
		for i in range(self.lineHandler1.getNumEntries()):
			entry = self.lineHandler1.getEntry(i)
			entries.append(entry)
		
		entries.sort(lambda x,y: cmp(y.getSurfacePoint(render).getZ(),x.getSurfacePoint(render).getZ()))

		if (len(entries)>0):
			self.speed *= 0.02
			self.angleTank = self.angleTank + elapsed*90.0
			self.napravlenie = 1
		else:
			self.speed = self.startSpeed
		#########################
		#########################
		# вычисление боковую коллизию танка 3
		entries = []
		for i in range(self.lineHandler3.getNumEntries()):
			entry = self.lineHandler3.getEntry(i)
			entries.append(entry)
		
		entries.sort(lambda x,y: cmp(y.getSurfacePoint(render).getZ(),x.getSurfacePoint(render).getZ()))

		if (len(entries)>0):
			self.speed *= 0.02
			self.napravlenie = -1
			self.angleTank = self.angleTank - elapsed*90.0
		else:
			self.speed = self.startSpeed
		#########################

		#####################################################################
		# вычисление высоты танка по поверхности
		startpos = self.node.getPos()
		entries = []
		for i in range(self.tankGroundHandler1.getNumEntries()):
			entry = self.tankGroundHandler1.getEntry(i)
			entries.append(entry)
		entries.sort(lambda x,y: cmp(y.getSurfacePoint(render).getZ(),x.getSurfacePoint(render).getZ()))

		if (len(entries)>0):
			Znew=entries[0].getSurfacePoint(render).getZ()
			deltaZ = (Znew - self.Zold)/1.2
			self.node.setZ(Znew-deltaZ)
			if (self.Zold - Znew)>0.1:
				self.node.setY(self.node, elapsed*self.speed)
			self.Zold = self.node.getZ()
		else:
			self.node.setPos(startpos)
		
		# самое интересное попытка наклонять танк повторяя поверхность по которой он движется - в разработке
		if entries:
			angleP=(180.0*entries[0].getInto().getNormal().getY())/pi
			#
			n = entries[0].getInto().getNormal()
			n.normalize() 
			q = Quat()
			q.setFromAxisAngle(self.angleTank,n)
			self.node.setQuat(q)
			self.node.setP(self.node, -angleP)
		
		# двигаем boxCam за танком
		self.boxCam.setPos(self.node.getPos())
		self.boxCam.setZ(self.node, 1)
		self.boxCam.setH(self.node.getH())
		#base.camera.lookAt(self.boxCam)
		#######################################################################
		
		if self.node.getX()>240 or self.node.getY()>240 or self.node.getX()<10 or self.node.getY()<10:
			self.angleTank = self.angleTank + (elapsed*60.0 * self.napravlenie)
		
		#выходим из обработчика
		return Task.cont